10 ' Program for barrel shells by folded plate analysis, Ketchum 1993
20 DEFDBL A-H, O-Z: DEFINT I-N
30 RESTORE 4000
40 CLS : PRINT
50 PRINT "INSTRUCTIONS FOR USE OF THIS PROGRAM": PRINT
60 PRINT "     This program solves for the stresses in barrel shells by using a folded"
70 PRINT "plate analogy based on the paper by DeFries-Skene and Scordeles:"
80 PRINT "Direct Stiffness Method for Folded Plates, PROCEENINGS, ASCE, 1964"
90 PRINT "pp.15-47.  Loads are placed at the nodes of the folded plate.  Any shape"
100 PRINT "may be used and the thickness may be varied.  The analysis follows the classic"
110 PRINT "methods of frame analysis. It is suggested that you immediatly RUN the"
120 PRINT "program to see the format and then examine the input in DATA statements"
130 PRINT "at Statements 4000 or 5000."
140 PRINT "     Because the a mistake in entering statement numbers may cause an "
150 PRINT "error in the program, you must be very carefull in entering data. Be"
160 PRINT "sure to retain a copy of the program that has not been touched."
170 PRINT "There are two examples shown.  The first is a simple symmetrical barrel"
180 PRINT "shell of constant thickness.  It is continuous with adjacent similar"
190 PRINT "shells.  The second example is a shell with an edge beam.  They may be"
200 PRINT "run by changing the RESTORE statement at Statement 290."
210 PRINT "     Another feature of the program is that a ribbed shell may be "
220 PRINT "simulated by altering the moment of inertia of the radial thickness to"
230 PRINT "give a new I value for linear foot. See the DIMENSIONS output. Also"
240 PRINT "by recycling through the analysis with the coordinates modified by"
250 PRINT "previous deflections, a nonlinear analysis can be obtained."
260 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
270 CLS : PRINT "BARREL SHELLS BY FOLDED PLATE ANALYSIS "
280 ' All input is from DATA statements at the end of the program,
290 RESTORE 5000
300 READ A$
310 PRINT : PRINT "JOB IDENTIFICATION: "; A$: PRINT
320 DEFINT I-N: READ SP, E: PRINT "SPAN = "; SP; "  MOD.ELAS. = "; E
330 READ NP, NJ: PRINT "NO. PLATES = "; NP; "  NO. OF JOINTS = "; NJ
340 DIM NC(NP, 8), IT(NP), ND(NJ, 4), SI(NP), MK(NJ, 2), TH(NP), CJ(NJ, 2)
350 DIM H(NP), V(NP), D(NP)
360 PRINT
370 INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
380 '
390 ' Main program, first part
400 GOSUB 770  ' Input subroutine
410 GOSUB 980  ' Equations, band width, code numbers
420 GOSUB 1410  ' Plate dimensions
430 GOSUB 1700  ' Loads
440 GOTO 460
450 '
460 GOSUB 3540  ' Zero Matricies
470 DIM DS(NJ, 2)
480 ' KQ = Flag for full assembly of stiffness matrix. 1 = diagonal
490 ' N  = Coefficent for Fourier series. (One term only.)
500 KQ = 0: N = 1
510 ' Make copies of coordinate matricies.
520 FOR I = 1 TO NJ: DS(I, 1) = CJ(I, 1): DS(I, 2) = CJ(I, 2): NEXT
530 ' Calculate dimensions of members.
540 FOR I = 1 TO NP: H(I) = CJ(I + 1, 1) - CJ(I, 1): V(I) = CJ(I + 1, 2) - CJ(I, 2)
550 D(I) = SQR(H(I) ^ 2 + V(I) ^ 2): NEXT
560 PRINT : PRINT "Assemble stiffness matrix"
570 ' Loop for stiffness matrix assembly.
580 FOR M = 1 TO NP
590 GOSUB 2720 ' Stiffness matrix
600 GOSUB 1830 ' Distribution
610 NEXT M
620 PRINT : PRINT "First part of solver": PRINT
630 GOSUB 3120  ' First part of solver
640 PRINT "Second part of solver"
650 GOSUB 3260  ' Backsubstitute
660 GOSUB 1930  ' Deflections
670 GOSUB 2210  ' Forces
680 GOSUB 2640  ' Statics check
690 PRINT : INPUT "<ENTER> FOR NEXT PASS OR R FOR NEW DATA "; ABC$
700 IF ABC$ = "R" THEN CLEAR : GOTO 10
710 '
720 GOSUB 2060  ' Revised Coordinates
730 GOSUB 3470  ' Copy loads and deflections
740 GOTO 540 ' Back to next pass
750 END
760 '
770 PRINT "MEMBER-JOINT RELATIONS ": PRINT "MEMBER  JT. TO JT."
780 READ NGM$: IF NGM$ = "N" THEN 800
790 FOR I = 1 TO NP: MK(I, 1) = I: MK(I, 2) = I + 1: NEXT I: GOTO 810
800 FOR I = 1 TO NP: READ II, MK(I, 1), MK(I, 2): NEXT
810 FOR I = 1 TO NP
820 PRINT USING "####  "; I; MK(I, 1); MK(I, 2)
830 NEXT
840 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
850 ' Set all deflection numbers to 1.
860 FOR I = 1 TO NJ: FOR J = 1 TO 4: ND(I, J) = 1: NEXT J, I
870 '
880 PRINT "SPECIAL SUPPORTS ": PRINT "JOINT     X    Y    Z  THETA"
890 READ I: IF I = 0 THEN 940
900 READ ND(I, 1), ND(I, 2), ND(I, 3), ND(I, 4)
910 PRINT USING "####  "; I; ND(I, 1); ND(I, 2); ND(I, 3); ND(I, 4)
920 GOTO 890
930 '
940 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
950 RETURN
960 '
970 ' Count the number of equations and set dimensions for forces etc.
980 N1 = 0
990 FOR I = 1 TO NJ
1000 FOR J = 1 TO 4
1010 IF ND(I, J) = 0 THEN 1030
1020 N1 = N1 + 1: ND(I, J) = N1
1030 NEXT J, I
1040 PRINT "THERE ARE "; N1; " EQUATIONS": PRINT
1050 DIM F(N1), D1(N1), F3(N1)
1060 '
1070 PRINT "EQUATION NUMBERS ": PRINT "JOINT  THETA   X   Y    Z"
1080 FOR I = 1 TO NJ
1090 PRINT USING "###  "; I; ND(I, 1); ND(I, 2); ND(I, 3); ND(I, 4)
1100 NEXT
1110 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
1120 '
1130 IB = 0: PRINT "CALCULATE BAND WIDTH "
1140 ' Select deflection numbers for each end of member,
1150 FOR M1 = 1 TO NP
1160 JN = MK(M1, 1): JM = MK(M1, 2): IR(1) = ND(JN, 1)
1170 IR(2) = ND(JN, 2): IR(3) = ND(JN, 3): IR(4) = ND(JN, 4)
1180 IR(5) = ND(JM, 1): IR(6) = ND(JM, 2): IR(7) = ND(JM, 3): IR(8) = ND(JM, 4)
1190 MI = 1000: MA = 0
1200 FOR I = 1 TO 8
1210 IF IR(I) = 0 THEN 1240
1220 IF IR(I) > MA THEN MA = IR(I)
1230 IF IR(I) < MI THEN MI = IR(I)
1240 NEXT I
1250 IC = MA - MI + 1
1260 PRINT IC;
1270 IF IC > IB THEN IB = IC
1280 NEXT M1
1290 PRINT : PRINT "BAND WIDTH = "; IB
1300 ' Set stiffness matrix size.
1310 DIM S(N1, IB)
1320 '
1330 FOR M1 = 1 TO NP
1340 JN = MK(M1, 1): JM = MK(M1, 2)
1350 FOR I = 1 TO 4
1360 NC(M1, I) = ND(JN, I): NC(M1, I + 4) = ND(JM, I)
1370 NEXT I, M1
1380 PRINT
1390 RETURN
1400 '
1410 PRINT "DIMENSIONS"
1420 PRINT "Use a 0 (zero) to indicate no revised moment of inertia"
1430 PRINT "A revised moment of inertia is in moment of inertia per foot."
1440 PRINT "PLATE    THICKNESS   REV. MOM. OF I."
1450 FOR I = 1 TO NP
1460 READ ID, TH(I), SI(I)
1470 PRINT USING " ##      ###.###       ###.###"; I; TH(I); SI(I)
1480 ' If value of moment of inertia is 0 (zero) then back to shell.
1490 IF SI(I) = 0 THEN SI(I) = TH(I) ^ 3 / 12
1500 NEXT
1510 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
1520 '
1530 READ NLS
1540 PRINT : PRINT "NO. OF MEMB WITH NO LONGIT. STIFFNESS = "; NLS
1550 IF NLS = 0 THEN 1580
1560 PRINT : PRINT "MEMBER NUMBERS: ";
1570 FOR I = 1 TO NLS: READ MLN(I): PRINT MLN(I); : NEXT: PRINT
1580 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
1590 '
1600 PRINT : PRINT "COORDINATES"
1610 PRINT "JOINT      X       Y"
1620 FOR I = 1 TO NJ
1630 READ ID, CJ(I, 1), CJ(I, 2)
1640 PRINT USING "  ##  ###.###   ###.###"; I; CJ(I, 1); CJ(I, 2)
1650 NEXT
1660 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
1670 RETURN
1680 '
1690 '
1700 PRINT "JOINT LOADS": PRINT "Positive directions are to the left and down."
1710 PRINT "JOINT    X LOAD    Y LOAD"
1720 READ I: IF I = 0 THEN 1770
1730 F(ND(I, 1)) = 0
1740 READ F(ND(I, 1)), F(ND(I, 2))
1750 PRINT USING "###  #,###.##   #,###.## "; I; F(ND(I, 1)); F(ND(I, 2))
1760 GOTO 1720
1770 PRINT : INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
1780 RETURN
1790 '
1800 '
1810 '
1820 ' Distribution subroutine.
1830 FOR I = 1 TO 8: IF NC(M, I) = 0 THEN 1880
1840 K = NC(M, I): FOR J = 1 TO 8: IF NC(M, J) = 0 THEN 1870
1850 L = NC(M, J): IF (K - L) > 0 THEN 1870
1860 L1 = L - K + 1: S(K, L1) = S(K, L1) + SP(I, J)
1870 NEXT J
1880 NEXT I
1890 RETURN
1900 '
1910 '
1920 ' Deflection subroutine.
1930 PRINT
1940 U = 1.05
1950 CLS : PRINT : PRINT "DEFLECTIONS, INCHES - PASS NO."; IG
1960 PRINT "JOINT     Y       ROTATION       X"
1970 FOR I = 1 TO NJ: PRINT USING "#### "; I;
1980 PRINT USING " ###.##### "; D1(ND(I, 2)) * 12 * U / E; D1(ND(I, 4)) * 12 * U / E;
1990 PRINT USING " ###.##### "; D1(ND(I, 1)) * 12 * U / E
2000 IF I = 20 THEN INPUT "<ENTER> TO CONTINUE "; ZZ: CLS : PRINT
2010 NEXT I
2020 RETURN
2030 '
2040 ' Subroutine for revised coordinates for previous deflectiions.
2050 ' Deduct deflections from old coorinates to give next pass.
2060 FOR I = 1 TO NJ: KX = ND(I, 1): KY = ND(I, 2)
2070 CJ(I, 1) = DS(I, 1) - D1(KX) / E: CJ(I, 2) = DS(I, 2) - D1(KY) / E
2080 NEXT
2090 '
2100 PRINT "REVISED COORDINATES - PASS NO. "; IG
2110 PRINT "JOINT      X           Y"
2120 FOR I = 1 TO NJ: PRINT USING "###  "; I;
2130 PRINT USING " ####.#### "; CJ(I, 1); CJ(I, 2)
2140 ' Allow only 20 members to print.
2150 IF I = 20 THEN INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
2160 NEXT
2170 PRINT : INPUT "<ENTER> TO CONTINUE "; Z: CLS
2180 RETURN
2190 '
2200 ' Subroutine to calculate and print stresses.
2210 KQ = 1
2220 PRINT : INPUT "<ENTER> TO CONTINUE "; Z: CLS
2230 PRINT : PRINT "STRESSES IN PLATES "
2240 HH$=" PLATE  SHEAR-PSI NEAR -> FAR         THRUST,PSI    STRESS, PSI   MOMENT FP."
2250 PRINT HH$
2260 FOR M = 1 TO NP
2270 ' Clear stress matrix
2280 FOR J = 1 TO 8: FOR JJ = 1 TO 8: SP(J, JJ) = 0: NEXT JJ, J
2290 ' Go to stiffness subroutine.
2300 GOSUB 2720
2310 ' Local deflection numbers.
2320 FOR I = 1 TO 8: IR(I) = NC(M, I): NEXT I
2330 ' Deflections for this member
2340 FOR I = 1 TO 8: F1(I) = D1(IR(I)): NEXT I
2350 ' Global stress = stiffness matrix x deflection.
2360 FOR I = 1 TO 8: S2(I) = 0: FOR J = 1 TO 8
2370 S2(I) = S2(I) + SP(I, J) * F1(J): NEXT J: NEXT I
2380 PRINT USING "###   ";M;
2390 ' Calculate stresses.
2400 ' SC, SD = Shear stresses convert stresses to PSI.
2410 COEF = PI / 2 / TH(M) / 144
2420 SC = S2(3) * COEF: SD = -S2(7) * COEF
2430 ' STL, STR = Thrust, each end
2440 COEF = 1 / D(M) / 12 / TH(M)
2450 STL = (S2(1) * H(M) + S2(2) * V(M)) * COEF
2460 STR = -(S2(5) * H(M) - S2(6) * V(M)) * COEF
2470 ' SA, SB = Longitudinal stress.
2480 COEF = 3.876 / SP / 144
2490 SA = -D1(NC(M, 3)) * COEF: SB = -D1(NC(M, 7)) * COEF
2500 ' SML = Moment at each end.
2510 SML = S2(4) * 1.05: SMR = -S2(8) * 1.05
2520 PRINT USING "  #,##,###.## "; SC;SD; STL; -SA;-SML
2530 IF M<NP THEN 2570
2540 PRINT "      FAR           ";
2550 PRINT USING "  #,##,###.## "; SD; STR; -SB;-SMR
2560 ' Place loads in stress matrix for statics check
2570 FOR I = 1 TO 8: F(NC(M, I)) = F(NC(M, I)) - S2(I): NEXT I
2580 VN = RM
2590 IF M = 20 THEN INPUT "<ENTER> TO CONTINUE "; ZZ: CLS:PRINT HH$
2600 NEXT M
2610 INPUT "<ENTER> TO CONTINUE "; ZZ: CLS
2620 RETURN
2630 '
2640 PRINT : PRINT "STATICS CHECK BY CODE NUMBERS"
2650 PRINT "UNBALANCED FORCES GREATER THAN .002"
2660 FOR I = 1 TO N1: IF F(I) > .002 THEN 2670 ELSE 2680
2670 PRINT "FORCE AT DEFLECTION NO. "; I; " IS "; F(I); ""
2680 NEXT
2690 PRINT : INPUT "<ENTER> TO CONTINUE "; Z: CLS : PRINT
2700 RETURN
2710 END
2720 ' Subroutine for stiffness
2730 N = 1
2740 D = D(M): PI = 3.141593: SI = SI(M): R = SI / D
2750 AS1 = 4 * R: B = 2 * R: CS = 6 * R / D: DS = 12 * R / (D * D)
2760 C1 = TH(M) * D * N * N * PI * PI / (4 * SP * SP)
2770 C2 = C1 / 3 + TH(M) / (2 * D): C3 = TH(M) / D
2780 C4 = -TH(M) * N * PI / (4 * SP): C5 = C1 / 2
2790 EP = C1 + C2: FP = C1 - C2: GP = EP: HP = C4
2800 TI = C3 + C5: TJ = C3 - C5: TL = V(M) / D
2810 TM = H(M) / D
2820 SP(1, 1) = DS * TL * TL + TI * TM * TM
2830 SP(1, 2) = DS * TM * TL - TI * TM * TL
2840 SP(1, 3) = -HP * TM: SP(1, 4) = -CS * TL
2850 SP(1, 5) = -DS * TL * TL - TJ * TM * TM
2860 SP(1, 6) = -DS * TL * TM + TJ * TL * TM
2870 SP(1, 7) = HP * TM: SP(1, 8) = -CS * TL
2880 SP(2, 2) = DS * TM * TM + TI * TL * TL
2890 SP(2, 3) = HP * TL: SP(2, 4) = -CS * TM
2900 SP(2, 5) = -DS * TM * TL + TJ * TM * TL
2910 SP(2, 6) = -DS * TM * TM - TJ * TL * TL
2920 SP(2, 7) = -HP * TL: SP(2, 8) = -CS * TM
2930 SP(4, 4) = AS1: SP(4, 5) = CS * TL
2940 SP(4, 6) = CS * TM: SP(4, 7) = 0: SP(4, 8) = B
2950 SP(3, 3) = EP: SP(3, 4) = 0: SP(3, 5) = SP(1, 3): SP(3, 6) = SP(2, 3)
2960 SP(3, 7) = FP: SP(3, 8) = 0
2970 SP(5, 5) = SP(1, 1): SP(5, 6) = SP(1, 2): SP(5, 7) = SP(1, 7)
2980 SP(5, 8) = -SP(1, 8): SP(6, 6) = SP(2, 2)
2990 SP(6, 7) = SP(2, 7): SP(6, 8) = -SP(2, 8): SP(7, 7) = GP: SP(7, 8) = 0: SP(8, 8) = AS1
3000 IF NLS = 0 THEN 3070
3010 FOR J = 1 TO NLS
3020 IF M = MLN(J) THEN 3040
3030 GOTO 3060
3040 FOR I = 1 TO 8: SP(I, 3) = 0: SP(3, I) = 0: SP(I, 7) = 0: SP(7, I) = 0
3050 NEXT I
3060 NEXT J
3070 IF KQ = 0 THEN RETURN
3080 K = 1: FOR I = 2 TO 8: FOR J = 1 TO K: SP(I, J) = SP(J, I)
3090 NEXT J: K = K + 1: NEXT I
3100 RETURN
3110 ' First part of solver
3120 FOR N = 1 TO N1
3130 FOR L = 2 TO IB
3140 C = S(N, L) / S(N, 1)
3150 I = N + L - 1
3160 IF (N1 - I) < 0 THEN 3240
3170 J = 0
3180 FOR K = L TO IB
3190 J = J + 1
3200 S(I, J) = S(I, J) - C * S(N, K)
3210 NEXT K
3220 S(N, L) = C
3230 NEXT L
3240 NEXT N
3250 RETURN
3260 '  Second part of solver
3270 FOR N = 1 TO N1
3280 FOR L = 2 TO IB
3290 I = N + L - 1
3300 IF (N1 - I) < 0 THEN 3330
3310 D1(I) = D1(I) - S(N, L) * D1(N)
3320 NEXT L
3330 D1(N) = D1(N) / S(N, 1)
3340 NEXT N
3350 N = N1
3360 N = N - 1
3370 IF (N) = 0 THEN 3440
3380 FOR K = 2 TO IB
3390 L = N + K - 1
3400 IF (N1 - L) < 0 THEN 3420
3410 D1(N) = D1(N) - S(N, K) * D1(L)
3420 NEXT K
3430 GOTO 3360
3440 RETURN
3450 '
3460 ' Copy loads and deflections.
3470 XX = 1: FOR I = 1 TO N1: F(I) = F3(I): D1(I) = F(I): NEXT
3480 ' Zero global stifness matrix
3490 FOR I = 1 TO N1: FOR J = 1 TO IB: S(I, J) = 0: NEXT J, I
3500 IG = IG + 1
3510 PRINT : PRINT "PASS NO. "; IG
3520 RETURN
3530 '
3540  ' Zero deflection and load matricies for second pass.
3550  FOR I = 1 TO N1: D1(I) = F(I): F3(I) = F(I): NEXT
3560 RETURN
3570 '
4000 DATA MOMENTS IN UNIFORM THICKNESS BARREL
4010 ' Span, E, No. of members, No. of joints
4020 DATA 60,432000000,8,9
4030 ' Generate member/joint relations (Y?N) ?
4040 DATA "N"
4050 ' Member/joint relations: Member, X joint, Y joint.
4060 DATA 1,1,2, 2,2,3, 3,3,4, 4,4,5, 5,5,6, 6,6,7, 7,7,8, 8,8,9
4070 ' Special supports: Joint, X, Y, Z, Phi
4080 DATA 1,0,1,1,0, 9,0,1,1,0, 0
4090 ' Member, thickness, Special I value, (0 for no special value)
4100 DATA 1,.333,0, 2,.333,0, 4,.333,0, 4,.333,0, 5,.333,0, 6,.333,0
4110 DATA 7,.333,0, 8,.333,0
4120 ' Do some members have no longitudinal stiffness, (List number)
4130 DATA 0
4140 ' Coordinates: Joint. X, Y
4150 DATA 1,0,0, 2,1.67,1.141, 3,3.398,2.145, 4,5.215,3.006, 5,7.095,3.717
4160 DATA 6,9.026,4.276, 7,10.996,4.677, 8,12.991,4.919, 9,15,5
4170 ' Loads: Joint, X, Y, (0 to Exit)
4180 DATA 2,0,161.6, 3,0,164, 4,0,166.1, 5,0,167.9, 6,0,169.1
4190 DATA 7,0,170, 8,0,170.7, 9,0,85.4, 0
5000 DATA SINGLE BARREL WITH EDGE BEAM
5010 ' Span, E, No. of members, No. of joints
5020 DATA 60,432E+6,13,14
5030 ' Generate member/joint relations (Y?N) ?
5040 DATA "Y"
5050 ' Member/joint relations: Member, X joint, Y joint.
5060 ' Special supports: Joint, X, Y, Z, Phi, (0 to exit)
5070 DATA 14,0,1,1,0, 0
5080 ' Member, thickness, Special I value, (0 for no special value)
5090 DATA 1,1.00,0, 2,.290,0, 4,.290,0, 4,.290,0, 5,.290,0, 6,.290,0
5100 DATA 7,.290,0, 8,.290,0, 9,.29,0, 10,.29,0, 11,.29,0, 12,.29,0
5110 DATA 13,.29,0
5120 ' Do some members have no longitudinal stiffness, (List number)
5130 DATA 0
5140 ' Coordinates: Joint. X, Y
5150 DATA 1,0,0, 2,0,3, 3,2.047,4.908, 4,4.339,6.520, 5,6.826,7.805
5160 DATA 6,9.465,8.741, 7,12.207,9.309, 8,15.000,9.500
5170 DATA 9,17.79,9.31, 10,20.53,8.74, 11,23.174,7.805, 12,25.66,6.52
5180 DATA 13,27.951,4.908, 14,30,3
5190 ' Loads: Joint, X, Y, (0 to Exit)
5200 DATA 1,0,225, 2,0,322, 3,0,194, 4,0,194, 5,0,194, 6,0,194
5210 DATA 7,0,194, 8,0,194, 9,0,194, 10,0,194, 11,0,194, 12,0,184, 0
